@@ -1,4 +1,5 @@
 {-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE CPP          #-}
 
 module Codec.CBOR.JSON
  ( encodeValue
@@ -13,11 +14,17 @@ import           Codec.CBOR.Encoding
 import           Codec.CBOR.Decoding
 import           Data.Aeson                          ( Value(..) )
 import qualified Data.Aeson                          as Aeson
-import qualified Data.HashMap.Lazy                   as HM
 import           Data.Scientific                     as Scientific
 import qualified Data.Text                           as T
 import qualified Data.Vector                         as V
 
+#if MIN_VERSION_aeson(2,0,0)
+import qualified Data.Aeson.Key                      as K
+import qualified Data.Aeson.KeyMap                   as KM
+#else
+import qualified Data.HashMap.Lazy                   as HM
+#endif
+
 -- | Encode a JSON value into CBOR.
 encodeValue :: Value -> Encoding
 encodeValue (Object vs) = encodeObject vs
@@ -31,8 +38,18 @@ encodeValue  Null       = encodeNull
 
 encodeObject :: Aeson.Object -> Encoding
 encodeObject vs =
-    encodeMapLen (fromIntegral (HM.size vs))
- <> HM.foldrWithKey (\k v r -> encodeString k <> encodeValue v <> r) mempty vs
+     encodeMapLen (fromIntegral size)
+  <> foldrWithKey (\k v r -> encodeString' k <> encodeValue v <> r) mempty vs
+  where
+#if MIN_VERSION_aeson(2,0,0)
+    size = KM.size vs
+    foldrWithKey = KM.foldrWithKey
+    encodeString' = encodeString . K.toText
+#else
+    size = HM.size vs
+    foldrWithKey = HM.foldrWithKey
+    encodeString' = encodeString
+#endif
 
 encodeArray :: Aeson.Array -> Encoding
 encodeArray vs =
@@ -58,7 +75,7 @@ decodeValue lenient = do
 
       TypeListLen      -> decodeListLen >>= decodeListN lenient
       TypeListLenIndef -> decodeListLenIndef >> decodeListIndef lenient []
-      TypeMapLen       -> decodeMapLen >>= flip (decodeMapN lenient) HM.empty
+      TypeMapLen       -> decodeMapLen >>= flip (decodeMapN lenient) mempty
 
       _           -> fail $ "unexpected CBOR token type for a JSON value: "
                          ++ show tkty
@@ -101,4 +118,10 @@ decodeMapN !lenient !n acc =
                  Bool   b | lenient -> return $ T.pack (show b)
                  _        -> fail "Could not decode map key type"
         !tv  <- decodeValue lenient
-        decodeMapN lenient (n-1) (HM.insert tk tv acc)
+        decodeMapN lenient (n-1) (insert tk tv acc)
+  where
+#if MIN_VERSION_aeson(2,0,0)
+    insert k v m = KM.insert (K.fromText k) v m
+#else
+    insert k v m = HM.insert k v m
+#endif
